@@ -19,7 +19,7 @@
 */
 /* PROCPS
    The procps suite is maintained as a suite by Charles Blake
-   <cblake@sdcc3.UCSD.EDU>.
+   <cblake@bbn.com>.
 */
    
 #include "proc/sysinfo.h"
@@ -51,7 +51,7 @@ void showheader(void);
 void getstat(unsigned *, unsigned *, unsigned *, unsigned long *,
 	     unsigned *, unsigned *, unsigned *, unsigned *,
 	     unsigned *, unsigned *, unsigned *);
-void getmeminfo(unsigned *, unsigned *, unsigned *);
+void getmeminfo(unsigned *, unsigned *, unsigned *, unsigned *);
 void getrunners(unsigned *, unsigned *, unsigned *);
 static char buff[BUFFSIZE]; /* used in the procedures */
 
@@ -61,7 +61,7 @@ static char buff[BUFFSIZE]; /* used in the procedures */
 
 int main(int argc, char *argv[]) {
 
-  const char format[]="%2u %1u %1u %5u %5u %5u %3u %3u %4u %4u %4u %4u %3u %3u %3u\n";
+  const char format[]="%2u %1u %1u %5u %5u %5u %5u %3u %3u %4u %4u %4u %4u %3u %3u %3u\n";
   unsigned int height=22; /* window height, reset later if needed. */
   unsigned long int args[2]={0,0};
   unsigned int moreheaders=TRUE;
@@ -70,25 +70,26 @@ int main(int argc, char *argv[]) {
   unsigned int running,blocked,swapped;
   unsigned int cpu_use[2], cpu_nic[2], cpu_sys[2];
   unsigned int duse,dsys,didl,div,divo2;
-  unsigned int memfree,membuff,swapused;
+  unsigned int memfree,membuff,swapused, memcache;
   unsigned long cpu_idl[2];
   unsigned int pgpgin[2], pgpgout[2], pswpin[2], pswpout[2];
   unsigned int inter[2],ticks[2],ctxt[2];
   unsigned int per=0, pero2; 
   unsigned long num=0;
   
+  set_linux_version();
   argc=0; /* redefined as number of integer arguments */
   for (argv++;*argv;argv++) {
     if ('-' ==(**argv)) {
-      if (*(++(*argv))=='V') {
+      switch (*(++(*argv))) {
+	case 'V':
 	display_version();
 	exit(0);
-      }
-      if (*(++(*argv))=='n') {
+      case 'n':
 	/* print only one header */
 	moreheaders=FALSE;
-      }
-      else {
+      break;
+      default:
 	/* no other aguments defined yet. */
 	usage();
       }
@@ -123,7 +124,7 @@ int main(int argc, char *argv[]) {
   pero2=(per/2);
   showheader();
   getrunners(&running,&blocked,&swapped);
-  getmeminfo(&memfree,&membuff,&swapused);
+  getmeminfo(&memfree,&membuff,&swapused,&memcache);
   getstat(cpu_use,cpu_nic,cpu_sys,cpu_idl,
 	  pgpgin,pgpgout,pswpin,pswpout,
 	  inter,ticks,ctxt);
@@ -135,7 +136,7 @@ int main(int argc, char *argv[]) {
   divo2= div/2;
   printf(format,
 	 running,blocked,swapped,
-	 swapused,memfree,membuff,
+	 swapused,memfree,membuff,memcache,
 	 (*(pswpin)*4*hz+divo2)/div,
 	 (*(pswpout)*4*hz+divo2)/div,
 	 (*(pgpgin)*hz+divo2)/div,
@@ -149,7 +150,7 @@ int main(int argc, char *argv[]) {
     if (moreheaders && ((i%height)==0)) showheader();
     tog= !tog;
     getrunners(&running,&blocked,&swapped);
-    getmeminfo(&memfree,&membuff,&swapused);
+    getmeminfo(&memfree,&membuff,&swapused,&memcache);
     getstat(cpu_use+tog,cpu_nic+tog,cpu_sys+tog,cpu_idl+tog,
 	  pgpgin+tog,pgpgout+tog,pswpin+tog,pswpout+tog,
 	  inter+tog,ticks+tog,ctxt+tog);
@@ -160,7 +161,7 @@ int main(int argc, char *argv[]) {
     divo2= div/2;
     printf(format,
 	   running,blocked,swapped,
-	   swapused,memfree,membuff,
+	   swapused,memfree,membuff,memcache,
 	   (((*(pswpin+tog)-*(pswpin+(!tog)))*4+pero2)/per),
 	   ((*(pswpout+tog)-*(pswpout+(!tog)))*4+pero2)/per,
 	   (*(pgpgin+tog)-*(pgpgin+(!tog))+pero2)/per,
@@ -175,7 +176,8 @@ int main(int argc, char *argv[]) {
 /**************************** others ***********************************/
 
 void usage(void) {
-  fprintf(stderr,"usage: %s [-n] [delay [count]]\n",PROGNAME);
+  fprintf(stderr,"usage: %s [-V] [-n] [delay [count]]\n",PROGNAME);
+  fprintf(stderr,"              -V prints version.\n");
   fprintf(stderr,"              -n causes the headers not to be reprinted regularly.\n");
   fprintf(stderr,"              delay is the delay between updates in seconds. \n");
   fprintf(stderr,"              count is the number of updates.\n");
@@ -199,10 +201,10 @@ int winhi(void) {
 
 
 void showheader(void){
-  printf("%6s%18s%8s%10s%10s%12s\n",
+  printf("%6s%24s%8s%10s%10s%12s\n",
 	 "procs","memory","swap","io","system","cpu");
-  printf(" %1s %1s %1s %5s %5s %5s %3s %3s %4s %4s %4s %4s %3s %3s %3s\n",
-	 "r","b","w","swpd","free","buff","si","so","bi","bo",
+  printf(" %1s %1s %1s %5s %5s %5s %5s %3s %3s %4s %4s %4s %4s %3s %3s %3s\n",
+	 "r","b","w","swpd","free","buff","cache","si","so","bi","bo",
 	 "in","cs","us","sy","id");
 }
 
@@ -235,12 +237,13 @@ void getstat(unsigned *cuse, unsigned *cice, unsigned *csys, unsigned long *cide
   }
 }
 
-void getmeminfo(unsigned *memfree, unsigned *membuff, unsigned *swapused) {
+void getmeminfo(unsigned *memfree, unsigned *membuff, unsigned *swapused, unsigned *memcache) {
   unsigned** mem;
   if (!(mem = meminfo())) crash("/proc/meminfo");
   *memfree  = mem[meminfo_main][meminfo_free]    >> 10;	/* bytes to k */
   *membuff  = mem[meminfo_main][meminfo_buffers] >> 10;
   *swapused = mem[meminfo_swap][meminfo_used]    >> 10;
+  *memcache = mem[meminfo_main][meminfo_cached]    >> 10;
 }
 
 void getrunners(unsigned int *running, unsigned int *blocked, 
